@@ -3,13 +3,15 @@ import pickle
 import json
 import os
 from matplotlib import pyplot as plt
+import numpy as np
+
 
 def load(path: os.PathLike, default=None):
     path = Path(path)
-    
+
     if not path.suffix in ('.pickle', '.json'):
         raise ValueError('path must end in either .pickle or .json.')
-    
+
     if path.exists():
         if path.suffix == '.pickle':
             with open(path, 'rb') as f:
@@ -33,7 +35,7 @@ def dump(value, path: os.PathLike):
             json.dump(value, f)
     else:
         raise ValueError('path must end in either .pickle or .json.')
-        
+
 
 def chunks(lst, n):
     """Yield successive n-sized chunks from lst."""
@@ -73,10 +75,45 @@ def imshow_grid(*images, nrows=None, ncols=None, titles=None, axes_off=True, fig
     plt.show()
 
 
+def mappend(lists, values):
+    for l, v in zip(lists, values):
+        l.append(v)
+
+
 def ema(ema_x, next_x, alpha=0.9):
     return alpha * ema_x + (1 - alpha) * next_x
 
 
-def mappend(lists, values):
-    for l, v in zip(lists, values):
-        l.append(v)
\ No newline at end of file
+def smooth_ema(X, alpha=0.9):
+    res = []
+    z = X[0]
+    for x in X:
+        z = ema(z, x, alpha)
+        res.append(z)
+    return np.array(res)
+
+
+def smooth_conv(y, box_pts, mode='valid'):
+    box = np.ones(box_pts) / box_pts
+    y_smooth = np.convolve(y, box, mode=mode)
+    return y_smooth
+
+
+def bchw_to_bhwc(x):
+    assert len(x.shape) == 4
+    if isinstance(x, np.ndarray):
+        return x.transpose(0, 2, 3, 1)
+    else:
+        return x.permute(0, 2, 3, 1)
+
+
+def bhwc_to_bchw(x):
+    assert len(x.shape) == 4
+    if isinstance(x, np.ndarray):
+        return x.transpose(0, 3, 1, 2)
+    else:
+        return x.permute(0, 3, 1, 2)
+
+
+def count_parameters(model, requires_grad_only=True):
+    return sum(p.numel() for p in model.parameters() if p.requires_grad or not requires_grad_only)
\ No newline at end of file
